/**
 * fylgja_command_handler.js
 * 
 * Unified command handler for the Fylgja Slack bot.
 * Processes natural language commands and routes to appropriate handlers.
 */
const logger = require('../utils/logger');
const { handleError } = require('../utils/error_handler');
const FILE_NAME = 'fylgja_command_handler.js';

// Import command handlers
const sigmaDetailsHandler = require('./sigma/sigma_details_handler');
const sigmaSearchHandler = require('./sigma/sigma_search_handler');
const sigmaCreateHandler = require('./sigma/sigma_create_handler');
const sigmaStatsHandler = require('./sigma/sigma_stats_handler');

// Import language processing utilities
const commandParser = require('../lang/command_parser');

/**
 * Handle the universal fylgja command
 * 
 * @param {Object} command - The Slack command object
 * @param {Function} respond - Function to send response back to Slack
 */
const handleCommand = async (command, respond) => {
  try {
    if (!command || !command.text) {
      logger.warn(`${FILE_NAME}: Empty command received for fylgja`);
      await respond({
        text: 'Please provide a command. Try `/fylgja help` for available commands.',
        response_type: 'ephemeral'
      });
      return;
    }

    logger.info(`${FILE_NAME}: Processing fylgja command: ${command.text}`);
    
    // Parse the natural language command
    const parsedCommand = await commandParser.parseCommand(command.text);
    
    if (!parsedCommand.success) {
      logger.warn(`${FILE_NAME}: Failed to parse command: ${command.text}`);
      await respond({
        text: `I couldn't understand that command. ${parsedCommand.message || ''}`,
        response_type: 'ephemeral'
      });
      return;
    }
    
    // Route to the appropriate handler based on the parsed command
    await routeCommand(parsedCommand.command, command, respond);
    
  } catch (error) {
    await handleError(error, `${FILE_NAME}: Command handler`, respond, {
      responseType: 'ephemeral'
    });
  }
};

/**
 * Route the command to the appropriate handler
 * 
 * @param {Object} parsedCommand - The parsed command object
 * @param {Object} originalCommand - The original Slack command object
 * @param {Function} respond - Function to send response back to Slack
 */
const routeCommand = async (parsedCommand, originalCommand, respond) => {
  const { action, module, params } = parsedCommand;
  
  // Create a modified command object with the extracted parameters
  const modifiedCommand = {
    ...originalCommand,
    text: params.join(' ')
  };
  
  // Log the routing decision
  logger.debug(`${FILE_NAME}: Routing command - Action: ${action}, Module: ${module}, Params: ${JSON.stringify(params)}`);
  
  // Route to the appropriate handler
  switch (`${module}:${action}`) {
    case 'sigma:details':
    case 'sigma:explain':
      await sigmaDetailsHandler.handleCommand(modifiedCommand, respond);
      break;
      
    case 'sigma:search':
      await sigmaSearchHandler.handleCommand(modifiedCommand, respond);
      break;
      
    case 'sigma:create':
      await sigmaCreateHandler.handleCommand(modifiedCommand, respond);
      break;
      
    case 'sigma:stats':
      await sigmaStatsHandler.handleCommand(modifiedCommand, respond);
      break;
      
    case 'help:general':
      await showHelp(respond);
      break;
      
    default:
      logger.warn(`${FILE_NAME}: Unknown command combination: ${module}:${action}`);
      await respond({
        text: `I don't know how to ${action} in ${module}. Try \`/fylgja help\` for available commands.`,
        response_type: 'ephemeral'
      });
  }
};

/**
 * Show help information
 * 
 * @param {Function} respond - Function to send response back to Slack
 */
const showHelp = async (respond) => {
  await respond({
    text: "Here are some example commands you can use with Fylgja:",
    blocks: [
      {
        type: "section",
        text: {
          type: "mrkdwn",
          text: "*Fylgja Commands*\nHere are some example commands you can use:"
        }
      },
      {
        type: "section",
        text: {
          type: "mrkdwn",
          text: "• `/fylgja explain rule from sigma where id=<rule_id>`\n• `/fylgja search sigma for <query>`\n• `/fylgja create rule in sigma with <parameters>`\n• `/fylgja show stats for sigma`"
        }
      }
    ],
    response_type: 'ephemeral'
  });
};

module.exports = {
  handleCommand
};